
/*
 * These are largely device specific operations for the audio devices. Some of
 * the calls are general, but may eventually be used by several of the SLab
 * applications.
 *
 * The operations for audio control are rather ugly. The application sets up
 * a number of values in the controlBuffer, and the engine intermittanly looks
 * to see if these values are initialised. If so, it acts on the values, and
 * then clears them. The reason we have to use this indirection is that the
 * front end applications do not own the audio device, this is under control
 * of the engine.
 *
 *	Jul 28th 96 - Only LINUX code is active.
 */

#ifdef DEBUG
#include <stdio.h>
#endif
#include <unistd.h>
#include <fcntl.h>
#ifdef SUBFRAGMENT
#include <malloc.h>
#endif

#ifdef linux
/*
 * This may want to be /usr/lib/oss/include/sys/soundcard.h if someone has
 * purchased the 4Front drivers?
 */
#include <sys/soundcard.h>
audio_buf_info bufferInfo;
#endif
#ifdef SUN
#include <sys/ioccom.h>
#include <sun/audioio.h>
#include <fcntl.h>

audio_info_t audioInfo;
#endif

/*
 * Audio device structure format definitions.
 */
#include "slabaudiodev.h"

static char *SLAB_SND_LABELS[SLAB_NRDEVICES] =	SLAB_DEVICE_LABELS;


/*
 * Setup audio parameters with device support.
 */
SL_setAudioDevParam2(audioDev, devID, param, left, right)
duplexDev *audioDev;
int param, devID;
short left, right;
{
	if ((devID < 0) || (devID >= MAX_DEVICES))
	{
		return;
	}
	if (audioDev->cflags & SLAB_NO_CONTROLS)
	{
		return;
	}
#ifdef DEBUG
	if (devID == 0)
		printf("SL_setAudioDevParam(mainOut, %i, l=%i, r=%i)\n",
			param, left, right);
	else
		printf("SL_setAudioDevParam(aux%i, %i, l=%i, r=%i)\n", devID,
			param, left, right);
#endif

	if (audioDev->mixerFD > 0)
	{
		SL_setAudioParameters(audioDev->mixerFD, param, left, right);
		return;
	}

	lockout(devID);

#ifdef DEBUG
	printf("SL_setAudioDevParam(%i, %x, %i, %i)\n", devID, param, left, right);
#endif

	if (((ALL_CAPABILITIES(devID) & (1 << (param & 0xffff))) == 0) &&
		(param != SLAB_REC_SRC))
	{
#ifdef DEBUG
		printf("Param not supported: id = %x, caps = %x, %x\n", param,
			audioDev->stereoCaps, audioDev->monoCaps);
#endif
		return;
	}

	audioDev->paramID = param;
	audioDev->paramValueLeft = left;
	audioDev->paramValueRight = right;
}

/*
 * This is a device specific routine. We take the fd, parameter, and left
 * right values, and configure the device. These routines should be used by the
 * engine to do the actual audio horsework.
 *
 * This is actually an internal routine, it should not be called from outside
 * of this file.
 */
SL_setAudioParameters2(fd, audioDev, param, valueL, valueR)
duplexDev *audioDev;
{
	int value, command;
#ifdef DEBUG
	printf("SL_setAudioParameters(%i, %i, %x, %x)\n",
		fd, param, valueL, valueR);
#endif

	switch(param) {
#ifdef linux
		case SLAB_REC_SRC:
		case SLAB_MM_CD:
		case SLAB_MM_MIC:
		case SLAB_MM_LINE:
		case SLAB_MM_LINE1:
		case SLAB_MM_LINE2:
		case SLAB_MM_LINE3:
		case SLAB_MM_SYNTH:
		case SLAB_MM_PCM:
		case SLAB_TREBLE:
		case SLAB_BASS:
		case SLAB_MID:
		case SLAB_INPUT_GAIN:
		case SLAB_VOL_CD:
		case SLAB_VOL_MIC:
		case SLAB_VOL_LINE:
		case SLAB_VOL_LINE1:
		case SLAB_VOL_LINE2:
		case SLAB_VOL_LINE3:
		case SLAB_VOL_SYNTH:
		case SLAB_VOL_PCM:
		default:
		/*
		 * These are all the rest, which should be from the microMixer, 
		 * scaled to 100 already.
		 */
			command = param & SLAB_MM_MASK;
			value = (valueR << 8) +
				valueL;
#else
#ifdef SUN
		case SLAB_INPUT_GAIN:
			audioInfo.record.gain = 
				 valueL * 2.56;
			break;
		case SLAB_MM_MIC:
		case SLAB_MM_LINE:
		case SLAB_VOL_MIC:
		case SLAB_VOL_LINE:
			audioInfo.play.gain = 
				 valueL * 2.56;
#endif /* SUN */
		default:
			;
#endif /* linux */
	}

#ifdef DEBUG
	printf("Command: %x, value: %x\n", command, value);
#endif

#ifdef linux
#ifdef IOCTL_DBG
	if (audioDev->cflags & SLAB_AUDIODBG)
		printf("ioctl(%i, MIXER_WRITE(%i), %i)\n", fd, command, value);
#endif
	if (ioctl(fd, MIXER_WRITE(command), &value)
		== -1) {
			printf("Error setting audio parameter %s\n",
				SLAB_SND_LABELS[command]);
		return(0);
	}
#else
#ifdef SUN
	if (ioctl(fd, AUDIO_SETINFO, &audioInfo)
		< 0)
		printf("Error setting audio parameter %s\n",
			SLAB_SND_LABELS[command]);
#endif
#endif

	return(0);
}

